@@ -5,7 +5,7 @@ export const layersDefaultProps: Record<string, unknown> = {
         id: "colormap-layer",
         pickable: true,
         visible: true,
-        valueRange: { type: "array" },
+        valueRange: { type: "array", value: [0, 1] },
         colorMapRange: { type: "array" },
         valueDecoder: {
             rgbScaler: [1, 1, 1],
@@ -15,6 +15,7 @@ export const layersDefaultProps: Record<string, unknown> = {
             step: 0,
         },
         rotDeg: 0,
+        colorMapName: "Rainbow",
     },
     Hillshading2DLayer: {
         "@@type": "Hillshading2DLayer",
@@ -24,11 +25,11 @@ export const layersDefaultProps: Record<string, unknown> = {
         pickable: true,
         visible: true,
         rotDeg: 0,
-        valueRange: { type: "array" },
-        colorMapRange: { type: "array" },
-        lightDirection: { type: "array", value: [1, 1, 1] },
-        ambientLightIntensity: { type: "number", value: 0.5 },
-        diffuseLightIntensity: { type: "number", value: 0.5 },
+        valueRange: { type: "array", value: [0, 1] },
+        colorMapRange: { type: "array", value: [0, 1] },
+        lightDirection: [1, 1, 1],
+        ambientLightIntensity: 0.5,
+        diffuseLightIntensity: 0.5,
         valueDecoder: {
             rgbScaler: [1, 1, 1],
             // By default, scale the [0, 256*256*256-1] decoded values to [0, 1]
@@ -37,24 +38,47 @@ export const layersDefaultProps: Record<string, unknown> = {
             step: 0,
         },
     },
+    MapLayer: {
+        "@@type": "MapLayer",
+        name: "Map",
+        id: "map3d-layer-float32",
+        pickable: true,
+        visible: true,
+        // Url to png image for height field.
+        mesh: "",
+        // Mesh error in meters. The output mesh is in higher resolution (more vertices) if the error is smaller.
+        meshMaxError: { type: "number", value: 5 },
+        // Url to png image for map properties. (ex, poro or perm values as a texture)
+        propertyTexture: "",
+        // Bounding box of the terrain image, [minX, minY, maxX, maxY] in world coordinates
+        bounds: { type: "array", value: null, false: true, compare: true },
+        colorMapRange: { type: "array" },
+        rotDeg: 0,
+        contours: [-1.0, -1.0],
+        // readout is default property value but if set to true it will be depth/z-value.
+        isReadoutDepth: true,
+        enableSmoothShading: true,
+    },
     Map3DLayer: {
         "@@type": "Map3DLayer",
         name: "Map 3D",
         id: "map3d-layer",
         pickable: true,
-        visible: false,
+        visible: true,
         // Url to png image for height field.
-        mesh: { type: "string", optional: false },
+        mesh: "",
+        meshValueRange: { type: "array", value: [0, 1] },
         // Mesh error in meters. The output mesh is in higher resolution (more vertices) if the error is smaller.
         meshMaxError: { type: "number", value: 5 },
         // Url to png image for map properties. (ex, poro or perm values as a texture)
-        propertyTexture: { type: "string", optional: false },
+        propertyTexture: "",
         // Bounding box of the terrain image, [minX, minY, maxX, maxY] in world coordinates
         bounds: { type: "array", value: null, false: true, compare: true },
-        valueRange: { type: "array", value: [0, 1] },
-        colorMapRange: { type: "array", value: [0, 1] },
+        propertyValueRange: { type: "array", value: [0, 1] },
         rotDeg: 0,
         contours: [-1.0, -1.0],
+        // If contour lines should follow depth or properties.
+        isContoursDepth: true,
         // readout is default property value but if set to true it will be depth/z-value.
         isReadoutDepth: false,
         enableSmoothShading: true,
@@ -72,7 +96,6 @@ export const layersDefaultProps: Record<string, unknown> = {
         name: "Wells",
         id: "wells-layer",
         autoHighlight: true,
-        selectionEnabled: true,
         opacity: 1,
         lineWidthScale: 5,
         pointRadiusScale: 8,
@@ -81,6 +104,7 @@ export const layersDefaultProps: Record<string, unknown> = {
         logCurves: true,
         refine: true,
         visible: true,
+        selectedWell: "@@#editedData.selectedWells", // used to get data from deckgl layer
     },
     FaultPolygonsLayer: {
         "@@type": "FaultPolygonsLayer",
@@ -97,7 +121,13 @@ export const layersDefaultProps: Record<string, unknown> = {
         id: "pie-layer",
         pickable: true,
         visible: true,
-        selectionEnabled: true,
+        selectedPie: "@@editedData.selectedPie", // used to get data from deckgl layer
+    },
+    AxesLayer: {
+        "@@type": "AxesLayer",
+        name: "Axes",
+        id: "axes-layer",
+        visible: true,
     },
     DrawingLayer: {
         "@@type": "DrawingLayer",
@@ -107,7 +137,7 @@ export const layersDefaultProps: Record<string, unknown> = {
         visible: true,
         mode: "drawLineString",
 
-        // Props mainly used to make the information available to the Map parent comp.
+        // Props used to get/set data in the drawing layer.
         selectedFeatureIndexes: [] as number[],
         data: {
             type: "FeatureCollection",
