@@ -1,10 +1,10 @@
-import { SimpleMeshLayer } from "@deck.gl/mesh-layers"; // XXX RENAME LAYER TIL NOE MED MESH??
+import { SimpleMeshLayer } from "@deck.gl/mesh-layers";
 import { SimpleMeshLayerProps } from "@deck.gl/mesh-layers/simple-mesh-layer/simple-mesh-layer";
 import { COORDINATE_SYSTEM } from "@deck.gl/core";
-import fsShader from "!!raw-loader!./terrainmap.fs.glsl";
+import fsShader from "./terrainmap.fs.glsl";
 import GL from "@luma.gl/constants";
 import { Texture2D } from "@luma.gl/core";
-import { DeckGLLayerContext } from "../../components/DeckGLWrapper";
+import { DeckGLLayerContext } from "../../components/Map";
 import { colorTablesArray, rgbValues } from "@emerson-eps/color-tables/";
 
 const DEFAULT_TEXTURE_PARAMETERS = {
@@ -14,19 +14,19 @@ const DEFAULT_TEXTURE_PARAMETERS = {
     [GL.TEXTURE_WRAP_T]: GL.CLAMP_TO_EDGE,
 };
 
-const DECODER = {
+export const DECODER = {
     rScaler: 256 * 256,
     gScaler: 256,
     bScaler: 1,
     offset: 0,
 };
 
-function getImageData(colorMapName: string, _colorTables: colorTablesArray) {
+function getImageData(colorMapName: string, colorTables: colorTablesArray) {
     const data = new Uint8Array(256 * 3);
 
     for (let i = 0; i < 256; i++) {
         const value = i / 255.0;
-        const rgb = rgbValues(value, colorMapName, _colorTables);
+        const rgb = rgbValues(value, colorMapName, colorTables);
         let color: number[] = [];
         if (rgb != undefined) {
             if (Array.isArray(rgb)) {
@@ -35,7 +35,6 @@ function getImageData(colorMapName: string, _colorTables: colorTablesArray) {
                 color = [rgb.r, rgb.g, rgb.b];
             }
         }
-        //console.log(color)
         data[3 * i + 0] = color[0];
         data[3 * i + 1] = color[1];
         data[3 * i + 2] = color[2];
@@ -56,11 +55,14 @@ export interface TerrainMapLayerProps<D> extends SimpleMeshLayerProps<D> {
     // Contourlines reference point and interval.
     contours: [number, number];
 
+    // Contourlines may be calculated either on depth/z-value or on property/texture value
+    isContoursDepth: boolean;
+
     // Name of color map.
     colorMapName: string;
 
     // Min and max property values.
-    valueRange: [number, number];
+    propertyValueRange: [number, number];
 
     // Use color map in this range.
     colorMapRange: [number, number];
@@ -70,16 +72,16 @@ export interface TerrainMapLayerProps<D> extends SimpleMeshLayerProps<D> {
 }
 
 const defaultProps = {
-    data: [{ position: [0, 0], angle: 0, color: [255, 0, 0] }], // dummy data
+    data: [{ position: [0, 0], angle: 0, color: [255, 0, 0, 0] }], // dummy data
 
     getPosition: (d: DataItem) => d.position,
     getColor: (d: DataItem) => d.color,
     getOrientation: (d: DataItem) => [0, d.angle, 0],
     contours: [-1, -1],
     colorMapName: "",
-    valueRange: [0.0, 1.0],
-    colorMapRange: [0.0, 1.0],
+    propertyValueRange: [0.0, 1.0],
     isReadoutDepth: false,
+    isContoursDepth: true,
     coordinateSystem: COORDINATE_SYSTEM.CARTESIAN,
     material: {
         ambient: 0.35,
@@ -102,9 +104,10 @@ export default class TerrainMapLayer extends SimpleMeshLayer<
         const contourReferencePoint = this.props.contours[0] ?? -1.0;
         const contourInterval = this.props.contours[1] ?? -1.0;
         const isReadoutDepth = this.props.isReadoutDepth;
+        const isContoursDepth = this.props.isContoursDepth;
 
-        const valueRangeMin = this.props.valueRange[0] ?? 0.0;
-        const valueRangeMax = this.props.valueRange[1] ?? 1.0;
+        const valueRangeMin = this.props.propertyValueRange[0] ?? 0.0;
+        const valueRangeMax = this.props.propertyValueRange[1] ?? 1.0;
 
         // If specified color map will extend from colorMapRangeMin to colorMapRangeMax.
         // Otherwise it will extend from valueRangeMin to valueRangeMax.
@@ -132,6 +135,7 @@ export default class TerrainMapLayer extends SimpleMeshLayer<
                 contourReferencePoint,
                 contourInterval,
                 isReadoutDepth,
+                isContoursDepth,
             },
         });
     }
@@ -180,8 +184,8 @@ export default class TerrainMapLayer extends SimpleMeshLayer<
         let value = (r + g + b) * (isPropertyReadout ? floatScaler : 1.0);
 
         if (isPropertyReadout) {
-            // Remap the [0, 1] decoded value to colorMapRange.
-            const [min, max] = this.props.colorMapRange;
+            // Remap the [0, 1] decoded value to property value range.
+            const [min, max] = this.props.propertyValueRange;
             value = value * (max - min) + min;
         }
 
