@@ -3,52 +3,47 @@ import { ExtendedLayerProps } from "../utils/layerTools";
 import { GeoJsonLayer, PathLayer } from "@deck.gl/layers";
 import { RGBAColor } from "@deck.gl/core/utils/color";
 import { subtract, distance, dot } from "mathjs";
-import { interpolateRgbBasis } from "d3-interpolate";
-import { color } from "d3-color";
+import {
+    rgbValues,
+    colorTableData,
+    colorTablesArray,
+} from "@emerson-eps/color-tables/";
 import {
     Feature,
     GeometryCollection,
     LineString,
+    Point,
     Position,
     FeatureCollection,
+    GeoJsonProperties,
+    Geometry,
 } from "geojson";
 import {
     LayerPickInfo,
     PropertyDataType,
     createPropertyData,
 } from "../utils/layerTools";
-import { patchLayerProps } from "../utils/layerTools";
 import { splineRefine } from "./utils/spline";
 import { interpolateNumberArray } from "d3";
 import { Position2D } from "@deck.gl/core/utils/positions";
+import { layersDefaultProps } from "../layersDefaultProps";
+import { UpdateStateInfo } from "@deck.gl/core/lib/layer";
+import { DeckGLLayerContext } from "../../components/Map";
 
 export interface WellsLayerProps<D> extends ExtendedLayerProps<D> {
     pointRadiusScale: number;
     lineWidthScale: number;
     outline: boolean;
-    selectedFeature: string;
-    selectionEnabled: boolean;
+    selectedWell: string;
     logData: string | LogCurveDataType;
     logName: string;
+    logColor: string;
     logrunName: string;
     logRadius: number;
     logCurves: boolean;
     refine: boolean;
 }
 
-const defaultProps = {
-    name: "Wells",
-    autoHighlight: true,
-    selectionEnabled: true,
-    opacity: 1,
-    lineWidthScale: 5,
-    pointRadiusScale: 8,
-    outline: true,
-    logRadius: 6,
-    logCurves: true,
-    refine: true,
-};
-
 export interface LogCurveDataType {
     header: {
         name: string;
@@ -69,7 +64,7 @@ export interface LogCurveDataType {
 }
 
 export interface WellsPickInfo extends LayerPickInfo {
-    logName?: string;
+    logName: string;
 }
 
 export default class WellsLayer extends CompositeLayer<
@@ -77,17 +72,33 @@ export default class WellsLayer extends CompositeLayer<
     WellsLayerProps<FeatureCollection>
 > {
     onClick(info: WellsPickInfo): boolean {
-        if (!this.props.selectionEnabled) {
+        // Disable selection when drawing is enabled
+        if (
+            this.context.layerManager.getLayers({
+                layerIds: ["drawing-layer"],
+            })?.[0].props.mode != "view"
+        ) {
             return false;
         }
 
-        patchLayerProps<FeatureCollection>(this, {
-            ...this.props,
-            selectedFeature: (info.object as Feature).properties?.["name"],
-        } as WellsLayerProps<FeatureCollection>);
+        (this.context as DeckGLLayerContext).userData.setEditedData({
+            selectedWell: (info.object as Feature).properties?.["name"],
+        });
         return true;
     }
 
+    shouldUpdateState({
+        changeFlags,
+    }: UpdateStateInfo<
+        WellsLayerProps<FeatureCollection<Geometry, GeoJsonProperties>>
+    >): boolean | string | null {
+        return (
+            changeFlags.viewportChanged ||
+            changeFlags.propsOrDataChanged ||
+            changeFlags.updateTriggersChanged
+        );
+    }
+
     renderLayers(): (GeoJsonLayer<Feature> | PathLayer<LogCurveDataType>)[] {
         if (!(this.props.data as FeatureCollection).features) {
             return [];
@@ -98,13 +109,16 @@ export default class WellsLayer extends CompositeLayer<
             ? splineRefine(this.props.data as FeatureCollection) // smooth well paths.
             : (this.props.data as FeatureCollection);
 
+        const is3d = this.context.viewport.constructor.name === "OrbitViewport";
+        const positionFormat = is3d ? "XYZ" : "XY";
+
         const outline = new GeoJsonLayer<Feature>(
             this.getSubLayerProps<Feature>({
                 id: "outline",
                 data,
                 pickable: false,
                 stroked: false,
-                positionFormat: "XY",
+                positionFormat,
                 pointRadiusUnits: "pixels",
                 lineWidthUnits: "pixels",
                 pointRadiusScale: this.props.pointRadiusScale,
@@ -119,7 +133,7 @@ export default class WellsLayer extends CompositeLayer<
                 data,
                 pickable: true,
                 stroked: false,
-                positionFormat: "XY",
+                positionFormat,
                 pointRadiusUnits: "pixels",
                 lineWidthUnits: "pixels",
                 pointRadiusScale: this.props.pointRadiusScale - 1,
@@ -135,11 +149,11 @@ export default class WellsLayer extends CompositeLayer<
                 id: "highlight",
                 data: getWellObjectByName(
                     data.features,
-                    this.props.selectedFeature
+                    this.props.selectedWell
                 ),
                 pickable: false,
                 stroked: false,
-                positionFormat: "XY",
+                positionFormat,
                 pointRadiusUnits: "pixels",
                 lineWidthUnits: "pixels",
                 pointRadiusScale: this.props.pointRadiusScale + 2,
@@ -153,7 +167,7 @@ export default class WellsLayer extends CompositeLayer<
             this.getSubLayerProps<LogCurveDataType>({
                 id: "log_curve",
                 data: this.props.logData,
-                positionFormat: "XY",
+                positionFormat,
                 pickable: true,
                 widthScale: 10,
                 widthMinPixels: 1,
@@ -161,13 +175,41 @@ export default class WellsLayer extends CompositeLayer<
                 getPath: (d: LogCurveDataType): Position[] =>
                     getLogPath(data.features, d, this.props.logrunName),
                 getColor: (d: LogCurveDataType): RGBAColor[] =>
-                    getLogColor(d, this.props.logrunName, this.props.logName),
+                    getLogColor(
+                        d,
+                        this.props.logrunName,
+                        this.props.logName,
+                        this.props.logColor,
+                        (this.context as DeckGLLayerContext).userData
+                            .colorTables
+                    ),
                 getWidth: (d: LogCurveDataType): number | number[] =>
                     this.props.logRadius ||
                     getLogWidth(d, this.props.logrunName, this.props.logName),
                 updateTriggers: {
-                    getColor: [this.props.logName],
-                    getWidth: [this.props.logName, this.props.logRadius],
+                    getColor: [
+                        this.props.logrunName,
+                        this.props.logName,
+                        (this.context as DeckGLLayerContext).userData
+                            .colorTables,
+                        this.props.logName,
+                        this.props.logColor,
+                    ],
+                    getWidth: [
+                        this.props.logrunName,
+                        this.props.logName,
+                        this.props.logRadius,
+                    ],
+                    getPath: [positionFormat],
+                },
+                onDataLoad: (value: LogCurveDataType[]) => {
+                    this.setState({
+                        legend: getLegendData(
+                            value,
+                            this.props.logName,
+                            this.props.logColor
+                        ),
+                    });
                 },
             })
         );
@@ -200,9 +242,16 @@ export default class WellsLayer extends CompositeLayer<
             this.props.logName
         );
 
+        // Patch for inverting tvd readout to fix issue #830,
+        // should make proper fix when handling z increase direction - issue #842
+        const inverted_tvd_property = tvd_property && {
+            ...tvd_property,
+            value: (tvd_property?.value as number) * -1,
+        };
+
         const layer_properties: PropertyDataType[] = [];
         if (md_property) layer_properties.push(md_property);
-        if (tvd_property) layer_properties.push(tvd_property);
+        if (inverted_tvd_property) layer_properties.push(inverted_tvd_property);
         if (log_property) layer_properties.push(log_property);
 
         return {
@@ -214,7 +263,9 @@ export default class WellsLayer extends CompositeLayer<
 }
 
 WellsLayer.layerName = "WellsLayer";
-WellsLayer.defaultProps = defaultProps;
+WellsLayer.defaultProps = layersDefaultProps[
+    "WellsLayer"
+] as WellsLayerProps<FeatureCollection>;
 
 //================= Local help functions. ==================
 
@@ -234,7 +285,7 @@ function getLogMd(d: LogCurveDataType, logrun_name: string): number[] {
     return log_id >= 0 ? getColumn(d.data, log_id) : [];
 }
 
-function getLogValues(
+export function getLogValues(
     d: LogCurveDataType,
     logrun_name: string,
     log_name: string
@@ -245,7 +296,7 @@ function getLogValues(
     return log_id >= 0 ? getColumn(d.data, log_id) : [];
 }
 
-function getLogInfo(
+export function getLogInfo(
     d: LogCurveDataType,
     logrun_name: string,
     log_name: string
@@ -270,20 +321,28 @@ function getWellObjectByName(
 ): Feature | undefined {
     return wells_data?.find(
         (item) =>
-            item.properties?.["name"].toLowerCase() === name?.toLowerCase()
+            item.properties?.["name"]?.toLowerCase() === name?.toLowerCase()
     );
 }
 
-function getWellCoordinates(well_object: Feature): Position[] {
+function getWellHeadCoordinates(well_object?: Feature): Position {
     return (
-        (well_object.geometry as GeometryCollection)?.geometries.find(
+        (well_object?.geometry as GeometryCollection)?.geometries.find(
+            (item) => item.type == "Point"
+        ) as Point
+    )?.coordinates;
+}
+
+function getWellCoordinates(well_object?: Feature): Position[] {
+    return (
+        (well_object?.geometry as GeometryCollection)?.geometries.find(
             (item) => item.type == "LineString"
         ) as LineString
     )?.coordinates;
 }
 
-function getWellMds(well_object: Feature): number[] {
-    return well_object.properties?.["md"][0];
+function getWellMds(well_object?: Feature): number[] {
+    return well_object?.properties?.["md"][0];
 }
 
 function getNeighboringMdIndices(mds: number[], md: number): number[] {
@@ -297,11 +356,16 @@ function getLogPath(
     logrun_name: string
 ): Position[] {
     const well_object = getWellObjectByName(wells_data, d.header.well);
-    if (well_object == undefined) return [];
-
     const well_xyz = getWellCoordinates(well_object);
     const well_mds = getWellMds(well_object);
-    if (well_xyz.length == 0 || well_mds.length == 0) return [];
+
+    if (
+        well_xyz == undefined ||
+        well_mds == undefined ||
+        well_xyz.length == 0 ||
+        well_mds.length == 0
+    )
+        return [];
 
     const log_xyz: Position[] = [];
     const log_mds = getLogMd(d, logrun_name);
@@ -334,36 +398,63 @@ function getLogIndexByNames(d: LogCurveDataType, names: string[]): number {
     return -1;
 }
 
-const color_interp = interpolateRgbBasis(["red", "yellow", "green", "blue"]);
 function getLogColor(
     d: LogCurveDataType,
     logrun_name: string,
-    log_name: string
+    log_name: string,
+    logColor: string,
+    colorTables: colorTablesArray
 ): RGBAColor[] {
     const log_data = getLogValues(d, logrun_name, log_name);
     const log_info = getLogInfo(d, logrun_name, log_name);
-    if (log_data.length == 0 || log_info == undefined) return [];
 
+    if (log_data.length == 0 || log_info == undefined) return [];
     const log_color: RGBAColor[] = [];
     if (log_info.description == "continuous") {
         const min = Math.min(...log_data);
         const max = Math.max(...log_data);
         const max_delta = max - min;
         log_data.forEach((value) => {
-            const rgb = color(color_interp((value - min) / max_delta))?.rgb();
+            const rgb = rgbValues(
+                (value - min) / max_delta,
+                logColor,
+                colorTables
+            );
             if (rgb != undefined) {
-                log_color.push([rgb.r, rgb.g, rgb.b]);
+                if (Array.isArray(rgb)) {
+                    log_color.push([rgb[0], rgb[1], rgb[2]]);
+                } else {
+                    log_color.push([rgb.r, rgb.g, rgb.b]);
+                }
             }
         });
     } else {
+        const colorsArray: [number, number, number, number][] = colorTableData(
+            logColor,
+            colorTables
+        );
+
         const log_attributes = getDiscreteLogMetadata(d, log_name)?.objects;
+        // eslint-disable-next-line
+        const attributesObject: { [key: string]: any } = {};
+        Object.keys(log_attributes).forEach((key) => {
+            // get the code from log_attributes
+            const code = log_attributes[key][1];
+            // compare the code and first value from colorsArray(colortable)
+            const colorArrays = colorsArray.find((value: number[]) => {
+                return value[0] == code;
+            });
+            if (colorArrays)
+                attributesObject[key] = [
+                    [colorArrays[1], colorArrays[2], colorArrays[3]],
+                    code,
+                ];
+        });
         log_data.forEach((log_value) => {
-            const dl_attrs = Object.entries(log_attributes).find(
+            const dl_attrs = Object.entries(attributesObject).find(
                 ([, value]) => value[1] == log_value
             )?.[1];
-            dl_attrs
-                ? log_color.push(dl_attrs[0])
-                : log_color.push([0, 0, 0, 0]);
+            dl_attrs ? log_color.push(dl_attrs[0]) : log_color.push([0, 0, 0]);
         });
     }
     return log_color;
@@ -398,12 +489,16 @@ function distToSegmentSquared(v: Position, w: Position, p: Position): number {
 
 // Interpolates point closest to the coords on trajectory
 function interpolateDataOnTrajectory(
-    coord: Position2D,
+    coord: Position,
     data: number[],
-    trajectory2D: Position[]
+    trajectory: Position[]
 ): number {
+    // if number of data points in less than 1 or
+    // length of data and trajectory are different we cannot interpolate.
+    if (data.length <= 1 || data.length != trajectory.length) return -1;
+
     // Identify closest well path leg to coord.
-    const segment_index = getSegmentIndex(coord, trajectory2D);
+    const segment_index = getSegmentIndex(coord, trajectory);
 
     const index0 = segment_index;
     const index1 = index0 + 1;
@@ -413,8 +508,8 @@ function interpolateDataOnTrajectory(
     const data1 = data[index1];
 
     // Get the nearest survey points.
-    const survey0 = trajectory2D[index0];
-    const survey1 = trajectory2D[index1];
+    const survey0 = trajectory[index0];
+    const survey1 = trajectory[index1];
 
     const dv = distance(survey0, survey1) as number;
     if (dv === 0) {
@@ -433,22 +528,31 @@ function interpolateDataOnTrajectory(
     return data0 * (1.0 - scalar_projection) + data1 * scalar_projection;
 }
 
-function getMd(coord: Position2D, feature: Feature): number | null {
-    if (!feature.properties || !feature.geometry) return null;
+function getMd(coord: Position, feature: Feature): number | null {
+    if (!feature.properties?.["md"]?.[0] || !feature.geometry) return null;
 
     const measured_depths = feature.properties["md"][0] as number[];
+    const trajectory3D = getWellCoordinates(feature);
 
-    const gc = feature.geometry as GeometryCollection;
-    const trajectory3D = (gc.geometries[1] as LineString).coordinates;
-    const trajectory2D = trajectory3D.map((v) => {
-        return v.slice(0, 2);
-    }) as Position[];
+    if (trajectory3D == undefined) return null;
 
-    return interpolateDataOnTrajectory(coord, measured_depths, trajectory2D);
+    let trajectory;
+    // In 2D view coord is of type Position2D and in 3D view it's Position3D,
+    // so use apropriate trajectory for interpolation
+    if (coord.length == 2) {
+        const trajectory2D = trajectory3D.map((v) => {
+            return v.slice(0, 2);
+        }) as Position[];
+        trajectory = trajectory2D;
+    } else {
+        trajectory = trajectory3D;
+    }
+
+    return interpolateDataOnTrajectory(coord, measured_depths, trajectory);
 }
 
 function getMdProperty(
-    coord: Position2D,
+    coord: Position,
     feature: Feature
 ): PropertyDataType | null {
     const md = getMd(coord, feature);
@@ -459,23 +563,34 @@ function getMdProperty(
     return null;
 }
 
-function getTvd(coord: Position2D, feature: Feature): number | null {
+function getTvd(coord: Position, feature: Feature): number | null {
     const trajectory3D = getWellCoordinates(feature);
-    if (trajectory3D == undefined) return null;
 
-    const trajectory2D = trajectory3D?.map((v) => {
-        return v.slice(0, 2);
-    }) as Position[];
+    // if trajectory is not found or if it has a data single point then get tvd from well head
+    if (trajectory3D == undefined || trajectory3D?.length <= 1) {
+        const wellhead_xyz = getWellHeadCoordinates(feature);
+        return wellhead_xyz?.[2] ?? null;
+    }
+    let trajectory;
+    // For 2D view coord is Position2D and for 3D view it's Position3D
+    if (coord.length == 2) {
+        const trajectory2D = trajectory3D?.map((v) => {
+            return v.slice(0, 2);
+        }) as Position[];
+        trajectory = trajectory2D;
+    } else {
+        trajectory = trajectory3D;
+    }
 
     const tvds = trajectory3D.map((v) => {
         return v[2];
     }) as number[];
 
-    return interpolateDataOnTrajectory(coord, tvds, trajectory2D);
+    return interpolateDataOnTrajectory(coord, tvds, trajectory);
 }
 
 function getTvdProperty(
-    coord: Position2D,
+    coord: Position,
     feature: Feature
 ): PropertyDataType | null {
     const tvd = getTvd(coord, feature);
@@ -491,7 +606,7 @@ function getTvdProperty(
 }
 
 // Identify closest path leg to coord.
-function getSegmentIndex(coord: Position2D, path: Position[]): number {
+function getSegmentIndex(coord: Position, path: Position[]): number {
     let min_d = Number.MAX_VALUE;
     let segment_index = 0;
     for (let i = 0; i < path?.length - 1; i++) {
@@ -560,3 +675,43 @@ function getLogProperty(
         );
     } else return null;
 }
+
+// Return data required to build welllayer legend
+function getLegendData(
+    logs: LogCurveDataType[],
+    logName: string,
+    logColor: string
+) {
+    const logInfo = getLogInfo(logs[0], logs[0].header.name, logName);
+    const title = "Wells / " + logName;
+    const legendProps = [];
+    if (logInfo?.description == "discrete") {
+        const meta = logs[0]["metadata_discrete"];
+        const metadataDiscrete = meta[logName].objects;
+        legendProps.push({
+            title: title,
+            colorName: logColor,
+            discrete: true,
+            metadata: metadataDiscrete,
+            valueRange: [],
+        });
+        return legendProps;
+    } else {
+        const minArray: number[] = [];
+        const maxArray: number[] = [];
+        logs.forEach(function (log: LogCurveDataType) {
+            const logValues = getLogValues(log, log.header.name, logName);
+            minArray.push(Math.min(...logValues));
+            maxArray.push(Math.max(...logValues));
+        });
+        legendProps.push({
+            title: title,
+            name: logName,
+            colorName: logColor,
+            discrete: false,
+            metadata: { objects: {} },
+            valueRange: [Math.min(...minArray), Math.max(...maxArray)],
+        });
+        return legendProps;
+    }
+}
