@@ -1,6 +1,5 @@
 import { configureStore, EnhancedStore } from "@reduxjs/toolkit";
 import { rootReducer } from "./reducer";
-import { enhancer } from "addon-redux";
 
 export type MapState = ReturnType<typeof rootReducer>;
 export const createStore: (
@@ -9,7 +8,6 @@ export const createStore: (
     configureStore({
         reducer: rootReducer,
         preloadedState: { spec: initialState },
-        enhancers: [enhancer],
         middleware: (getDefaultMiddleware) =>
             getDefaultMiddleware({
                 serializableCheck: {
