@@ -3,7 +3,7 @@ import WellLogViewer from "./WellLogViewer";
 
 export default {
     component: WellLogViewer,
-    title: "WellLogViewer/Demo",
+    title: "WellLogViewer/Demo/WellLogViewer",
     argTypes: {
         id: {
             description:
@@ -22,6 +22,14 @@ export default {
             description: "Orientation of the track plots on the screen.",
             defaultValue: false,
         },
+        hideTitles: {
+            description: "Hide titles on the tracks.",
+            defaultValue: false,
+        },
+        hideLegend: {
+            description: "Hide legends on the tracks.",
+            defaultValue: false,
+        },
         readoutOptions: {
             description:
                 "Options for readout panel.<br/>" +
@@ -41,9 +49,39 @@ export default {
     },
 };
 
+function fillInfo(controller) {
+    if (!controller) return "-";
+    const baseDomain = controller.getContentBaseDomain();
+    const domain = controller.getContentDomain();
+    const selection = controller.getContentSelection();
+    return (
+        "total: [" +
+        baseDomain[0].toFixed(0) +
+        ", " +
+        baseDomain[1].toFixed(0) +
+        "], " +
+        "visible: [" +
+        domain[0].toFixed(0) +
+        ", " +
+        domain[1].toFixed(0) +
+        "]" +
+        (selection[0] !== undefined
+            ? ", selected: [" +
+              selection[0].toFixed(0) +
+              (selection[1] !== undefined
+                  ? ", " + selection[1].toFixed(0)
+                  : "") +
+              "]"
+            : "")
+    );
+}
+
 const Template = (args) => {
+    const infoRef = React.useRef();
+    const setInfo = function (info) {
+        if (infoRef.current) infoRef.current.innerHTML = info;
+    };
     const [controller, setController] = React.useState(null);
-    const [info, setInfo] = React.useState("");
     const onCreateController = React.useCallback(
         (controller) => {
             setController(controller);
@@ -51,33 +89,10 @@ const Template = (args) => {
         [controller]
     );
     const onContentRescale = React.useCallback(() => {
-        if (!controller) {
-            setInfo("-");
-            return;
-        }
-        const baseDomain = controller.getContentBaseDomain();
-        const domain = controller.getContentDomain();
-        const selection = controller.getContentSelection();
-        setInfo(
-            "total: [" +
-                baseDomain[0].toFixed(0) +
-                ", " +
-                baseDomain[1].toFixed(0) +
-                "], " +
-                "visible: [" +
-                domain[0].toFixed(0) +
-                ", " +
-                domain[1].toFixed(0) +
-                "]" +
-                (selection[0] !== undefined
-                    ? ", selected: [" +
-                      selection[0].toFixed(0) +
-                      (selection[1] !== undefined
-                          ? ", " + selection[1].toFixed(0)
-                          : "") +
-                      "]"
-                    : "")
-        );
+        setInfo(fillInfo(controller));
+    }, [controller]);
+    const onContentSelection = React.useCallback(() => {
+        setInfo(fillInfo(controller));
     }, [controller]);
 
     return (
@@ -90,9 +105,10 @@ const Template = (args) => {
                     {...args}
                     onCreateController={onCreateController}
                     onContentRescale={onContentRescale}
+                    onContentSelection={onContentSelection}
                 />
             </div>
-            <div style={{ width: "100%", flex: 0 }}>{info}</div>
+            <div ref={infoRef} style={{ width: "100%", flex: 0 }}></div>
         </div>
     );
 };
@@ -101,25 +117,28 @@ export const Example1Vertical = Template.bind({});
 Example1Vertical.args = {
     id: "Well-Log-Viewer",
     horizontal: false,
-    welllog: require("../../../demo/example-data/L898MUD.json"),
+    hideTitles: false,
+    hideLegend: false,
+    welllog: require("../../../demo/example-data/L898MUD.json")[0],
     template: require("../../../demo/example-data/welllog_template_1.json"),
     colorTables: require("../../../demo/example-data/color-tables.json"),
 };
 
-export const Example1Template2 = Template.bind({});
-Example1Template2.args = {
+export const Example2Horizontal = Template.bind({});
+Example2Horizontal.args = {
     id: "Well-Log-Viewer2",
     horizontal: true,
-    welllog: require("../../../demo/example-data/L898MUD.json"),
+    welllog:
+        require("../../../demo/example-data/WL_RAW_AAC-BHPR-CAL-DEN-GR-MECH-NEU-NMR-REMP_MWD_3.json")[0],
     template: require("../../../demo/example-data/welllog_template_2.json"),
     colorTables: require("../../../demo/example-data/color-tables.json"),
 };
 
-export const Example2Vertical = Template.bind({});
-Example2Vertical.args = {
+export const Discrete = Template.bind({});
+Discrete.args = {
     id: "Well-Log-Viewer3",
     horizontal: false,
-    welllog: require("../../../demo/example-data/WL_RAW_AAC-BHPR-CAL-DEN-GR-MECH-NEU-NMR-REMP_MWD_3.json"),
-    template: require("../../../demo/example-data/welllog_template_1.json"),
+    welllog: require("../../../demo/example-data/volve_logs.json")[0],
+    template: require("../../../demo/example-data/welllog_template_2.json"),
     colorTables: require("../../../demo/example-data/color-tables.json"),
 };
